/*
                   Network Interface Stats Fetching

 */

#ifndef IFSTATS_H
#define IFSTATS_H


#include <sys/types.h>

#ifdef IFSTATS_USE_LINUX_KERNEL_PPP
# include <sys/socket.h>
# include <linux/in.h>
# include <linux/if.h>
# include <linux/ppp_defs.h>
# include <linux/if_ppp.h>
#endif


/*
 *	Default interface name
 */
#ifndef DEF_INTERFACE_NAME
# define DEF_INTERFACE_NAME	"ppp0"
#endif


/* For IP addresses. */
typedef union {
        u_int32_t whole;
        u_int8_t charaddr[4];	/* Break down. */
} if_ip_addr_union;



/*
 *	Interface stats descriptor structure:
 */
typedef struct {

	unsigned long	disgarded_frames,
			rx_bytes,
			sx_bytes,
			rex_bytes,
			sex_bytes,
			rx_packets,
			sx_packets,
			rx_lrqs,
			sx_lrqs;

	if_ip_addr_union ip;	/* Interface's IP address in network
				 * byte order.
				 */

	int bandwidth;	/* Bytes per second. */

	time_t time;	/* Time stamp, in milliseconds since midnight. */


	/* Private. */
#ifdef IFSTATS_USE_LINUX_KERNEL_PPP
	char *interface;		/* Name of interface. */
	int socket;
        struct ifreq ifr;
        struct ppp_stats ppp_info[1];
        struct pppstat	*ppp_st_ptr,
			last_ppp_st[1];
#else
	char *interface;		/* Name of interface. */

	int stats_file_is_virtual;	/* Indicates stats_file is in 
					 * memory, ie /proc/<*> files.
					 */
	char *stats_file;		/* Interface statistics file. */
#endif

} if_stats_desc_struct;


extern int IFSIsUp(if_stats_desc_struct *ifsd);
extern int IFSGetIP(if_stats_desc_struct *ifsd);
extern int IFSGetBandwidth(if_stats_desc_struct *ifsd);

extern int IFSInit(if_stats_desc_struct *ifsd, const char *interface);
extern int IFSFetch(if_stats_desc_struct *ifsd);
extern void IFSShutdown(if_stats_desc_struct *ifsd);



#endif	/* IFSTATS_H */
