/*
               Interface Monitor Window: Callback Handlers

	Functions:

	int IMWMenuCB(void *ptr, int op_code)

        int IMWConnectPBCB(void *ptr)
        int IMWDisconnectPBCB(void *ptr)
	int IMWExitPBCB(void *ptr)
	int IMWStatsPBCB(void *ptr)

	---

 */

#include <stdio.h>
#include <sys/types.h>

#include "../include/disk.h" 
#include "../include/string.h"
#include "../include/strexp.h"
#include "../include/prochandle.h"

#include "../include/osw-x.h"
#include "../include/widget.h"

#include "xifmon.h"



int IMWMenuCB(void *ptr, int op_code)
{
	int status = 0;
	ifmon_win_struct *imw;


	imw = (ifmon_win_struct *)ptr;
	if(imw == NULL)
	    return(-1);



	switch(op_code)
	{
	  case IMW_ACTION_NONE:
	    break;

	  case IMW_ACTION_CONNECT:
	    status = IMWConnectPBCB(imw);
	    break;

	  case IMW_ACTION_DISCONNECT:
	    status = IMWDisconnectPBCB(imw);
	    break;

	  case IMW_ACTION_EXIT:
	    status = IMWExitPBCB(imw);
	    break;

	  case IMW_ACTION_MAP_STATS:
	    status = IMWStatsPBCB(imw);
	    break;
	}

	return(status);
}


/*
 *	Procedure to run connect script.
 */
int IMWConnectPBCB(void *ptr)
{
	ifmon_win_struct *imw;  
	pid_t pid;


        imw = (ifmon_win_struct *)ptr;
        if(imw == NULL)
            return(-1);


	/* Skip if already connected or connecting... */
	if(imw->if_state != IF_STATE_OFF)
	    return(0);


	/* Redraw if state lights. */
	imw->if_state = IF_STATE_STARTUP;
	IMWDraw(imw, IMW_DRAW_AMOUNT_IFSTATE);


	/* Run connect script. */
	pid = Exec(imw->connect_script);


	return(0);
}

int IMWDisconnectPBCB(void *ptr)
{
        ifmon_win_struct *imw;
        pid_t pid;
            

        imw = (ifmon_win_struct *)ptr;
        if(imw == NULL)
            return(-1);


        /* Redraw if state lights. */
        imw->if_state = IF_STATE_OFF;
        IMWDraw(imw, IMW_DRAW_AMOUNT_IFSTATE);


        /* Run disconnect script. */
        pid = Exec(imw->disconnect_script);


        return(0);
}

int IMWExitPBCB(void *ptr)
{
        ifmon_win_struct *imw;
            

        imw = (ifmon_win_struct *)ptr;
        if(imw == NULL)
            return(-1);


	runlevel = 1;

        return(0);
}

int IMWStatsPBCB(void *ptr)
{
        ifmon_win_struct *imw;


        imw = (ifmon_win_struct *)ptr;
        if(imw == NULL)
            return(-1);

	SWMap(&imw->stats_win);


	return(0);
}
