/*

	Functions:

	void XIFMHandleSignal(int s)
	void XIFMResetTimmers()

	int XIFMInit(int argc, char *argv[])
	void XIFMManage()
	void XIFMShutdown()

	---

 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>

#include "../include/string.h"
#include "../include/osw-x.h"
#include "../include/widget.h"

#include "xifmon.h"


#include "ct_data/bkg.h"
#include "ct_data/led_cursor.h"
#include "ct_data/if_light_cursor.h"
#include "ct_data/exit_label.h"
#include "ct_data/connect_label.h"
#include "ct_data/disconnect_label.h"
#include "ct_data/stats_label.h"

#include "ct_data/icon.h"



int runlevel;
time_t cur_millitime;

next_struct next;
xifmon_image_struct xifmon_image;
xifmon_font_struct xifmon_font;
xifmon_color_struct xifmon_color;

ifmon_win_struct ifmon_win;



/*
 *	Signal handler.
 */
void XIFMHandleSignal(int s)
{
	switch(s)
	{
	  case SIGINT:
	    runlevel = 1;
	    break;

	  case SIGTERM:
            runlevel = 1;
            break;

	  case SIGSEGV:
            runlevel = 1;
            break;

	  case SIGPIPE:
	    signal(SIGPIPE, XIFMHandleSignal);
	    break;
	}


	return;
}


void XIFMResetTimmers()
{
	next.if_stats = 0;
	next.if_status = 0;

	ifmon_win.cur_time = 0;
	ifmon_win.last_time = 0;
	ifmon_win.stats_win.last_update = 0;


	return;
}


int XIFMInit(int argc, char *argv[])
{
	int i, status;
	const char *arg;


        signal(SIGINT, XIFMHandleSignal);
        signal(SIGTERM, XIFMHandleSignal);
        signal(SIGSEGV, XIFMHandleSignal);
        signal(SIGPIPE, XIFMHandleSignal);

	XIFMResetTimmers();


        /* Parse arguments. */
        for(i = 1; i < argc; i++)
        {
	    arg = (const char *)argv[i];
            if(arg == NULL)
                continue;

	    /* Help: --help */
            if(strcasepfx(arg, "--h") ||
               strcasepfx(arg, "-h") ||
               strcasepfx(arg, "-?") ||
               strcasepfx(arg, "/?") ||
               strcasepfx(arg, "?")
	    )
	    {
		printf(PROG_USAGE_MESG);
		return(-4);
	    }
            /* Version: --version */
            else if(strcasepfx(arg, "--v") ||
                    strcasepfx(arg, "-v")
            )
            {
                printf(PROG_NAME " Version " PROG_VERSION "\n");
#ifdef IFSTATS_USE_LINUX_KERNEL_PPP
		printf("With direct Linux kernel ppp support.\n");
#endif	/* IFSTATS_USE_LINUX_KERNEL_PPP */
		printf(PROG_COPYRIGHT "\n");
                return(-4);
            }
	}


        /* Connect to GUI. */
        status = OSWGUIConnect(argc, argv);
        if(status)
            return(-1);

        /* Initialize widget globals. */
        status = WidgetInitGlobals(argc, argv);
        if(status)
            return(-1);

/*
        SBarSetNotifyFunction(???);
 */

	OSWLoadPixelRGB(
	    &xifmon_color.graph_bkg,
	    0x00, 0x00, 0x00
	);
        OSWLoadPixelRGB( 
            &xifmon_color.graph_grid,
            0x00, 0x40, 0x00
        );
        OSWLoadPixelRGB( 
            &xifmon_color.graph_plot,
            0x00, 0xb0, 0x00
        );
        OSWLoadPixelRGB( 
            &xifmon_color.graph_text,
            0x00, 0xff, 0x00
        );



	OSWLoadFont(
	    &xifmon_font.graph_font,
	    "6x12"
	);

        xifmon_image.icon = WidgetLoadImageFromTgaData(icon_tga);
	xifmon_image.bkg = WidgetLoadImageFromTgaData(bkg_tga);
        xifmon_image.led_cursor = WidgetLoadImageFromTgaData(led_cursor_tga);
        xifmon_image.if_light_cursor = WidgetLoadImageFromTgaData(if_light_cursor_tga);
        xifmon_image.exit_label = WidgetLoadImageFromTgaData(exit_label_tga);
        xifmon_image.connect_label = WidgetLoadImageFromTgaData(connect_label_tga);
        xifmon_image.disconnect_label = WidgetLoadImageFromTgaData(disconnect_label_tga);
        xifmon_image.stats_label = WidgetLoadImageFromTgaData(stats_label_tga);

	xifmon_image.icon_pm = OSWCreatePixmapFromImage(xifmon_image.icon);


	/* Initialize interface monitor window. */
	status = IMWInit(&ifmon_win, argc, argv);
	if(status)
            return(-1);

	IMWMap(&ifmon_win);


	return(0);
}

void XIFMManage()
{
	long t;
	event_t event;
	int events_handled = 0;


	/* Update timming. */
	t = MilliTime();
	if(t < cur_millitime)
	{
	    XIFMResetTimmers();
	}
	cur_millitime = t;


	/* Manage GUI events. */
	while(OSWEventsPending() > 0)
	{
	    OSWWaitNextEvent(&event);

            events_handled += WidgetManage(&event);

	    events_handled += IMWManage(&ifmon_win, &event);
	}
	/* Manage widgets if no events were handled above. */
	if(events_handled == 0)
	    WidgetManage(NULL);


	/* Manage schedualed events. */
	if(next.if_stats <= cur_millitime)
	{
	    /*   Time to get new stats from interface, redraw will
	     *   be done as needed.
	     */
	    IMWStatsFetch(&ifmon_win);

	    next.if_stats = cur_millitime + INT_IF_STATS;
	}

        if(next.if_status <= cur_millitime)
        {
            /*   Time to recheck status of interface, redraw will
	     *   be done as needed.
	     */
	    IMWStatsIFStatus(&ifmon_win);

            next.if_status = cur_millitime + INT_IF_STATUS;
        }


	return;
}

void XIFMShutdown()
{
	IMWDestroy(&ifmon_win);

        OSWDestroyImage(&xifmon_image.icon);
	OSWDestroyImage(&xifmon_image.bkg);
        OSWDestroyImage(&xifmon_image.led_cursor);
        OSWDestroyImage(&xifmon_image.if_light_cursor);
        OSWDestroyImage(&xifmon_image.exit_label);
        OSWDestroyImage(&xifmon_image.connect_label);
        OSWDestroyImage(&xifmon_image.disconnect_label);
        OSWDestroyImage(&xifmon_image.stats_label);

	OSWDestroyPixmap(&xifmon_image.icon_pm);

        OSWUnloadFont(&xifmon_font.graph_font);

	OSWDestroyPixel(&xifmon_color.graph_bkg);
        OSWDestroyPixel(&xifmon_color.graph_grid);
        OSWDestroyPixel(&xifmon_color.graph_plot);
        OSWDestroyPixel(&xifmon_color.graph_text);


        /*   Free widget globals after all client allocated widgets
         *   have been deallocated.
         */
        WidgetDestroyGlobals();

        /* Disconnect from the GUI. */
        OSWGUIDisconnect();   


	return;
}


int main(int argc, char *argv[])
{
	int status;


	runlevel = 1;

	status = XIFMInit(argc, argv);
	switch(status)
	{
	  case -1:
	    XIFMShutdown();
	    return(1);
	    break;

	  case -4:
	    XIFMShutdown();
	    return(0);
	    break;

	  case 0:
	    break;
	}


        runlevel = 2;
	while(runlevel >= 2)
	{
            XIFMManage();

            usleep(8000);
	}

	XIFMShutdown();


	return(0);
}
