/*
                          Stats Window: GUI Management

	Functions:

	int SWClosePBCB(void *ptr)

	int SWInit(stats_win_struct *sw, int argc, char *argv[])
	void SWDraw(stats_win_struct *sw, int amount);
	int SWManage(stats_win_struct *sw, event_t *event);
	void SWMap(stats_win_struct *sw);
	void SWUnmap(stats_win_struct *sw);
	void SWDestroy(stats_win_struct *sw);

	---

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "../include/disk.h"
#include "../include/string.h"

#include "../include/osw-x.h"
#include "../include/widget.h"

#include "xifmon.h"


/*
 *	Size constants (in pixels):
 */
#define SW_DEF_WIDTH		380
#define SW_DEF_HEIGHT		400

#define SW_PROMPT_HEIGHT	30

#define SW_BTN_WIDTH		70
#define SW_BTN_HEIGHT		28


/*
 *	Width of graph, in pixels.  This value also implies the number
 *	of x axis points across.
 */
#define SW_GRID_WIDTH		150


/*
 *	Close button callback.
 */
int SWClosePBCB(void *ptr)
{
	stats_win_struct *sw;

	sw = (stats_win_struct *)ptr;
	if(sw == NULL)
	    return(-1);

	SWUnmap(sw);


	return(0);
}


/*
 *	Initialize stats window.
 */
int SWInit(stats_win_struct *sw, int argc, char *argv[])
{
	int i, x, y;
	unsigned int width, height;
	char title[256];
	win_attr_t wattr;


	if(sw == NULL)
	    return(-1);


        /* Parse arguments. */
        for(i = 1; i < argc; i++)
        {
            if(argv[i] == NULL)
                continue;

	}

	width = SW_DEF_WIDTH;
	height = SW_DEF_HEIGHT;

        if(osw_gui[0].def_geometry_set)
        {
            x = osw_gui[0].def_toplevel_x;
            y = osw_gui[0].def_toplevel_y;
        }
        else
        {
            x = 0;
            y = 0;
        }

        sw->map_state = 0;
        sw->is_in_focus = 0;
        sw->x = x;
        sw->y = y; 
        sw->width = width;
        sw->height = height;

	sw->last_update = cur_millitime;


        /* Create toplevel. */
        if(
            OSWCreateWindow(
                &sw->toplevel,
                osw_gui[0].root_win,
                sw->x, sw->y,   
                sw->width, sw->height
            )
        )
            return(-1);      
         
        sw->toplevel_buf = 0;

	WidgetCenterWindow(sw->toplevel, WidgetCenterWindowToRoot);
	OSWGetWindowAttributes(sw->toplevel, &wattr);
	sw->x = wattr.x;
	sw->y = wattr.y;
	sw->width = wattr.width;
	sw->height = wattr.height;


	/* Use interface name in title. */
	sprintf(
	    title,
	    "Statistics for %s",
	    ((ifmon_win.interface == NULL) ? "Unknown Interface" :
		ifmon_win.interface)
	);
        OSWSetWindowWMProperties(
            sw->toplevel,
            title,		/* Title. */
            "Statistics",	/* Icon title. */
            xifmon_image.icon_pm,	/* Icon. */
            False,			/* Let WM set coordinates? */
            sw->x, sw->y,
            sw->width, sw->height,
            sw->width, sw->height,
            WindowFrameStyleFixed,
            NULL, 0
        );
        OSWSetWindowInput(
            sw->toplevel,
            OSW_EVENTMASK_TOPLEVEL
        );


	/* Graph. */
        if(
            OSWCreateWindow(
                &sw->graph_win,
                sw->toplevel,
                (int)sw->width - SW_GRID_WIDTH - 10, 10,
		SW_GRID_WIDTH, 100
            )
        )
            return(-1);
        OSWSetWindowInput(
            sw->graph_win,
            ExposureMask
        );

	/* Allocate graph points. */
	sw->total_graph_pts_rx = SW_GRID_WIDTH;
	sw->graph_pt_rx = (u_int8_t *)malloc(
	    sw->total_graph_pts_rx * sizeof(u_int8_t)
	);
	if(sw->graph_pt_rx == NULL)
	{
	    sw->total_graph_pts_rx = 0;
	}

        sw->total_graph_pts_sx = SW_GRID_WIDTH;
        sw->graph_pt_sx = (u_int8_t *)malloc(
            sw->total_graph_pts_sx * sizeof(u_int8_t)
        );
        if(sw->graph_pt_sx == NULL)
        {
            sw->total_graph_pts_sx = 0;
        }



	/* IP Prompt. */
	x = 10;
	y = 10 + 48 + 10;
	if(
	    PromptInit(
                &sw->ip_prompt,
                sw->toplevel,
                x, y,
                190,
                SW_PROMPT_HEIGHT,
                PROMPT_STYLE_FLUSHED,
                "Address:",
                64,
                0,
                NULL
            )
        )
            return(-1);

	/* Bandwidth prompt. */
        y += (SW_PROMPT_HEIGHT + 5);
        if(
            PromptInit(
                &sw->bandwidth_prompt,
                sw->toplevel,
                x, y,
                160,
                SW_PROMPT_HEIGHT,
                PROMPT_STYLE_FLUSHED,
                "Bandwidth:",
                64,
                0,
                NULL
            )
        )
            return(-1);   

        /* Total bytes sent. */
        y += (SW_PROMPT_HEIGHT + 10);
        if(
            PromptInit(
                &sw->total_sx_prompt,
                sw->toplevel,
                x, y,
                200,
                SW_PROMPT_HEIGHT,
                PROMPT_STYLE_FLUSHED,
                "    Total SX:",
                64,
                0,  
                NULL
            )
        )
            return(-1);
	/* Total bytes recieved. */
        y += (SW_PROMPT_HEIGHT + 5);
        if(
            PromptInit(
                &sw->total_rx_prompt, 
                sw->toplevel,
                x, y,
                200,
                SW_PROMPT_HEIGHT,
                PROMPT_STYLE_FLUSHED,
                "    Total RX:", 
                64,
                0,  
                NULL
            )
        )
            return(-1);

        /* Total errors sent. */
        y += (SW_PROMPT_HEIGHT + 10); 
        if(
            PromptInit(
                &sw->errors_sx_prompt,
                sw->toplevel,
                x, y,
                200,
                SW_PROMPT_HEIGHT,
                PROMPT_STYLE_FLUSHED,
                "   Errors SX:",
                64,
                0,   
                NULL
            )
        )
            return(-1);
        /* Total errors recieved. */
        y += (SW_PROMPT_HEIGHT + 5);
        if(
            PromptInit(
                &sw->errors_rx_prompt,
                sw->toplevel,
                x, y,
                200,
                SW_PROMPT_HEIGHT,
                PROMPT_STYLE_FLUSHED,
                "   Errors RX:",
                64,
                0,
                NULL
            )
        )
            return(-1);

        /* Total packets sent. */
        x = 220;
        y = 120;
        if(
            PromptInit(
                &sw->packets_sx_prompt,
                sw->toplevel,
                x, y,
                150,
                SW_PROMPT_HEIGHT,
                PROMPT_STYLE_FLUSHED,
                "Packets SX:",
                48,
                0,
                NULL
            )
        )
            return(-1);
        /* Total packets recieved. */
        y += (SW_PROMPT_HEIGHT + 5);
        if(
            PromptInit(
                &sw->packets_rx_prompt,
                sw->toplevel,
                x, y,
                150,
                SW_PROMPT_HEIGHT,
                PROMPT_STYLE_FLUSHED,
                "Packets RX:",
                48,
                0, 
                NULL
            )
        )
            return(-1);

        /* Total lrqs sent. */
        y += (SW_PROMPT_HEIGHT + 10);
        if(
            PromptInit(
                &sw->lrqs_sx_prompt,
                sw->toplevel,
                x, y,
                150,
                SW_PROMPT_HEIGHT,
                PROMPT_STYLE_FLUSHED,
                "   LRQS SX:",
                48,
                0,
                NULL 
            )
        )
            return(-1);
	/* Total lrqs recieved. */
        y += (SW_PROMPT_HEIGHT + 5);
        if(
            PromptInit(
                &sw->lrqs_rx_prompt,
                sw->toplevel,
                x, y,  
                150,
                SW_PROMPT_HEIGHT,
                PROMPT_STYLE_FLUSHED,
                "   LRQS RX:",
                48,
                0,
                NULL
            )
        )
            return(-1);

	/* Link prompts togeather. */
	sw->ip_prompt.next = &sw->bandwidth_prompt;
	sw->ip_prompt.prev = &sw->lrqs_rx_prompt;

	sw->bandwidth_prompt.next = &sw->total_sx_prompt;
        sw->bandwidth_prompt.prev = &sw->ip_prompt;

        sw->total_sx_prompt.next = &sw->total_rx_prompt;
        sw->total_sx_prompt.prev = &sw->bandwidth_prompt;

        sw->total_rx_prompt.next = &sw->errors_sx_prompt;
        sw->total_rx_prompt.prev = &sw->total_sx_prompt;

        sw->errors_sx_prompt.next = &sw->errors_rx_prompt;
        sw->errors_sx_prompt.prev = &sw->total_rx_prompt;

        sw->errors_rx_prompt.next = &sw->packets_sx_prompt;
        sw->errors_rx_prompt.prev = &sw->errors_sx_prompt;

        sw->packets_sx_prompt.next = &sw->packets_rx_prompt;
        sw->packets_sx_prompt.prev = &sw->errors_rx_prompt;

        sw->packets_rx_prompt.next = &sw->lrqs_sx_prompt;
        sw->packets_rx_prompt.prev = &sw->packets_sx_prompt;

        sw->lrqs_sx_prompt.next = &sw->lrqs_rx_prompt;
        sw->lrqs_sx_prompt.prev = &sw->packets_rx_prompt;

        sw->lrqs_rx_prompt.next = &sw->ip_prompt;
        sw->lrqs_rx_prompt.prev = &sw->lrqs_sx_prompt;


	/* Close button. */
	if(
	    PBtnInit(
                &sw->close_btn,
                sw->toplevel,
                (int)sw->width - 10 - SW_BTN_WIDTH,
		(int)sw->height - 10 - SW_BTN_HEIGHT,
                SW_BTN_WIDTH, SW_BTN_HEIGHT,
                "Close",
                PBTN_TALIGN_CENTER,
                NULL,
                sw,
                SWClosePBCB
            )
        )
            return(-1);



	return(0);
}

void SWDraw(stats_win_struct *sw, int amount)
{
	int x, y, avg;
	unsigned int width, height;
	image_t *image;
	u_int8_t *pt;
	int total_pts;

        win_attr_t wattr;
        font_t *prev_font;   
        pixmap_t pixmap;
        win_t w;
	char text[256];


        if(sw == NULL)
            return;

        /* Map as needed. */
        if(!sw->map_state)
        {
            OSWMapRaised(sw->toplevel);
            sw->map_state = 1;
            amount = SW_DRAW_AMOUNT_COMPLETE;


	    OSWMapRaised(sw->graph_win);

            PBtnMap(&sw->refresh_btn);
            PBtnMap(&sw->close_btn);

            PromptMap(&sw->ip_prompt);
            PromptMap(&sw->bandwidth_prompt);

            PromptMap(&sw->total_rx_prompt);
            PromptMap(&sw->total_sx_prompt);

            PromptMap(&sw->errors_rx_prompt);
            PromptMap(&sw->errors_sx_prompt);

            PromptMap(&sw->packets_rx_prompt);
            PromptMap(&sw->packets_sx_prompt);

            PromptMap(&sw->lrqs_rx_prompt);
            PromptMap(&sw->lrqs_sx_prompt);
        }


        /* Recreate buffers as needed. */
        if(sw->toplevel_buf == 0)
        {
            OSWGetWindowAttributes(sw->toplevel, &wattr);
            if(
                OSWCreatePixmap(
                    &sw->toplevel_buf,
                    wattr.width, wattr.height
                )
            )
                return;      
        }

        prev_font = OSWQueryCurrentFont();


        /* Draw background. */  
        if(amount == IMW_DRAW_AMOUNT_COMPLETE)
        {
            pixmap = sw->toplevel_buf;
            w = sw->toplevel;
            OSWGetWindowAttributes(w, &wattr);

	    if(widget_global.force_mono)
	    {
		OSWClearPixmap(
		    pixmap, wattr.width, wattr.height,
		    osw_gui[0].black_pix
		);
	    }
	    else
	    {
	        WidgetPutImageTile(
                    pixmap,
		    widget_global.std_bkg_img,
                    wattr.width, wattr.height
                );

		image = xifmon_image.icon;
		if(image != NULL)
		    WidgetPutImageRaised(
			pixmap, image,
			10, 10,
			16		/* Altitude. */
		    );

		/* Draw horizontal rule. */
		OSWSetFgPix(widget_global.surface_shadow_pix);
		y = (int)wattr.height - 25 - SW_BTN_HEIGHT;
		OSWDrawLine(
		    pixmap,
		    0, y,
		    wattr.width, y
		);
                OSWSetFgPix(widget_global.surface_highlight_pix);
                y = (int)wattr.height - 24 - SW_BTN_HEIGHT;
                OSWDrawLine(
                    pixmap,
                    0, y,
                    wattr.width, y
                );

	    }

            OSWPutBufferToWindow(w, pixmap);
        }    
 

	/* Graph. */
	if((amount == IMW_DRAW_AMOUNT_COMPLETE) ||
           (amount == SW_DRAW_AMOUNT_GRAPH)
	)
        {
	    pixmap = sw->toplevel_buf;
            w = sw->graph_win;
            OSWGetWindowAttributes(w, &wattr);


	    /* Redraw graph background. */
	    OSWClearPixmap(
		pixmap,
		wattr.width, wattr.height,
		xifmon_color.graph_bkg
	    );


            /* Redraw send graph. */
            pt = sw->graph_pt_sx;
            total_pts = sw->total_graph_pts_sx;
            width = wattr.width;
            height = (((int)wattr.height / 2) - 10);

            /* Draw send grids. */
            OSWSetFgPix(xifmon_color.graph_grid);
            OSWDrawLine(
                pixmap,
                0,
                5,
                (int)wattr.width,
                5
            );
            OSWDrawLine(
                pixmap,
                0,
                ((int)height / 2) + 5,
                (int)wattr.width,
                ((int)height / 2) + 5
            );

	    /* Draw send plot. */
            OSWSetFgPix(xifmon_color.graph_plot);
            for(x = 0, avg = 0; x < total_pts; x++)
            {
                y = (int)height * (int)pt[x] / 255;
		avg += (int)pt[x];

                OSWDrawLine(
                    pixmap,
                    x, (int)height + 5 - y,
                    x, (int)height + 5
                );
	    }
	    avg = avg / ((total_pts > 0) ? total_pts : 1);

            sprintf(text, "SX AVG: %.0f%%",
		(double)((double)avg / 255) * 100
	    );
	    OSWSetFgPix(xifmon_color.graph_text);
            OSWSetFont(xifmon_font.graph_font);
            OSWDrawString(
                pixmap,
                ((int)wattr.width / 2),
                5 + 9,
                text
            );


	    /* Redraw recieve graph. */
	    pt = sw->graph_pt_rx;
	    total_pts = sw->total_graph_pts_rx;
	    width = wattr.width;
	    height = (((int)wattr.height / 2) - 10);

	    /* Draw recieve grids. */
            OSWSetFgPix(xifmon_color.graph_grid);
            OSWDrawLine(
                pixmap,
                0,
		(int)height + 15,
                (int)wattr.width,
		(int)height + 15
	    );
            OSWDrawLine(
                pixmap, 
                0, 
                (int)height + ((int)height / 2) + 15,    
                (int)wattr.width,
                (int)height + ((int)height / 2) + 15
            );

	    /* Draw recieve plot. */
	    OSWSetFgPix(xifmon_color.graph_plot);
	    for(x = 0, avg = 0; x < total_pts; x++)
	    {
		y = (int)height * (int)pt[x] / 255;
		avg += (int)pt[x];

		OSWDrawLine(
		    pixmap,
		    x, (int)height + (int)height + 15 - y,
		    x, (int)height + (int)height + 15
		);
	    }
            avg = avg / ((total_pts > 0) ? total_pts : 1);

            sprintf(text, "RX AVG: %.0f%%",
                (double)((double)avg / 255) * 100  
            );
            OSWSetFgPix(xifmon_color.graph_text);
	    OSWSetFont(xifmon_font.graph_font);
            OSWDrawString(
		pixmap,
		((int)wattr.width / 2),
		(int)height + (15 + 9),
		text
	    );


	    /* Draw frame. */
	    if(!widget_global.force_mono)
	        WidgetFrameButtonPixmap(
		    pixmap, True,
		    wattr.width, wattr.height,
		    widget_global.surface_highlight_pix,
		    widget_global.surface_shadow_pix
	        );

	    OSWPutBufferToWindow(w, pixmap);
	}


	OSWSetFont(prev_font);



	return;
}

int SWManage(stats_win_struct *sw, event_t *event)
{
        int events_handled = 0;


        if((sw == NULL) ||
           (event == NULL)
        )
            return(events_handled);

        if(!sw->map_state &&
           (event->type != MapNotify)
        )
            return(events_handled);


        switch(event->type)
        {
          case Expose:
            if(event->xany.window == sw->toplevel)
            {
                SWDraw(sw, SW_DRAW_AMOUNT_COMPLETE);

                events_handled++;
                return(events_handled);
            }
	    else if(event->xany.window == sw->graph_win)
	    {
                SWDraw(sw, SW_DRAW_AMOUNT_GRAPH);
             
                events_handled++;   
                return(events_handled);
            }
            break;

          case FocusIn:
            if(event->xany.window == sw->toplevel)
            {
                sw->is_in_focus = 1;

                events_handled++;
                return(events_handled);
            }
            break;

          case FocusOut:
            if(event->xany.window == sw->toplevel)
            {
                sw->is_in_focus = 0;

                events_handled++;
                return(events_handled);
            }
            break;

          case MapNotify:
            if(event->xany.window == sw->toplevel)
            {
                if(!sw->map_state)
                    SWMap(sw);

                events_handled++;
                return(events_handled);
            }
            break;

          case UnmapNotify:
            if(event->xany.window == sw->toplevel)
            {
                if(sw->map_state)
                    SWUnmap(sw);
         
                events_handled++;
                return(events_handled);
            }
            break;

          case ClientMessage:
            if(OSWIsEventDestroyWindow(sw->toplevel, event))
            {
		SWUnmap(sw);

                events_handled++;
                return(events_handled);
            }
            break;
	}


	/* Manage buttons. */
	if(events_handled == 0)
	    events_handled += PBtnManage(&sw->refresh_btn, event);

        if(events_handled == 0)
            events_handled += PBtnManage(&sw->close_btn, event);


	/* Manage prompts. */
	if(events_handled == 0)
            events_handled += PromptManage(&sw->ip_prompt, event);
        if(events_handled == 0)
            events_handled += PromptManage(&sw->bandwidth_prompt, event);

        if(events_handled == 0)
            events_handled += PromptManage(&sw->total_rx_prompt, event);
        if(events_handled == 0)
            events_handled += PromptManage(&sw->total_sx_prompt, event);

        if(events_handled == 0)
            events_handled += PromptManage(&sw->errors_rx_prompt, event);
        if(events_handled == 0)
            events_handled += PromptManage(&sw->errors_sx_prompt, event);

        if(events_handled == 0)
            events_handled += PromptManage(&sw->packets_rx_prompt, event);
        if(events_handled == 0)
            events_handled += PromptManage(&sw->packets_sx_prompt, event);

        if(events_handled == 0)
            events_handled += PromptManage(&sw->lrqs_rx_prompt, event);
        if(events_handled == 0)
            events_handled += PromptManage(&sw->lrqs_sx_prompt, event);




        return(events_handled);
}


void SWMap(stats_win_struct *sw)
{
        if(sw == NULL)
            return;

	sw->map_state = 0;
	SWDraw(sw, SW_DRAW_AMOUNT_COMPLETE);

	return;
}


void SWUnmap(stats_win_struct *sw)
{
        if(sw == NULL)
            return;

	PromptUnmap(&sw->ip_prompt);
	PromptUnmap(&sw->bandwidth_prompt);

        PromptUnmap(&sw->total_rx_prompt);              
        PromptUnmap(&sw->total_sx_prompt);

        PromptUnmap(&sw->errors_rx_prompt);  
        PromptUnmap(&sw->errors_sx_prompt);

        PromptUnmap(&sw->packets_rx_prompt);             
        PromptUnmap(&sw->packets_rx_prompt);

        PromptUnmap(&sw->lrqs_rx_prompt);
        PromptUnmap(&sw->lrqs_sx_prompt);


	PBtnUnmap(&sw->refresh_btn);
        PBtnUnmap(&sw->close_btn);


	OSWUnmapWindow(sw->graph_win);
	OSWUnmapWindow(sw->toplevel);
	sw->map_state = 0;

	OSWDestroyPixmap(&sw->toplevel_buf);
}

void SWDestroy(stats_win_struct *sw)
{
	if(sw == NULL)
	    return;


        free(sw->graph_pt_rx);
        sw->graph_pt_rx = NULL;

        sw->total_graph_pts_rx = 0;


	free(sw->graph_pt_sx);
	sw->graph_pt_sx = NULL;

	sw->total_graph_pts_sx = 0;


	if(IDC())
	{
	    PromptDestroy(&sw->ip_prompt);
            PromptDestroy(&sw->bandwidth_prompt);

            PromptDestroy(&sw->total_rx_prompt);
            PromptDestroy(&sw->total_sx_prompt);

            PromptDestroy(&sw->errors_rx_prompt);
            PromptDestroy(&sw->errors_sx_prompt);

            PromptDestroy(&sw->packets_rx_prompt);
            PromptDestroy(&sw->packets_sx_prompt);

            PromptDestroy(&sw->lrqs_rx_prompt);
            PromptDestroy(&sw->lrqs_sx_prompt);

	    PBtnDestroy(&sw->refresh_btn);
            PBtnDestroy(&sw->close_btn);

	    OSWDestroyWindow(&sw->graph_win);

            OSWDestroyPixmap(&sw->toplevel_buf);
            OSWDestroyWindow(&sw->toplevel);
	}


	sw->map_state = 0;
	sw->is_in_focus = 0;
	sw->x = 0;
	sw->y = 0;
	sw->width = 0;
	sw->height = 0;

	sw->last_update = 0;

	return;
}
