/*
                    X I N T E R F A C E   M O N I T O R

	Formerly known as XPPPConsole (xpppcon).

	Copyright (C) 1997 - 2001 WolfPack Entertainment

 */

#ifndef XIFMON_H
#define XIFMON_H

#include <sys/time.h>

#include "../include/osw-x.h"
#include "../include/widget.h"


/*
 *	Default interface:
 */
#ifndef DEF_INTERFACE_NAME
# define DEF_INTERFACE_NAME	"ppp0"
#endif

/*
 *	Default bandwidth (in bits per second):
 */
#define DEF_BANDWIDTH	28800

/*
 *	Minimum link bandwidth (in bits per second):
 */
#define BANDWIDTH_MIN	1200


/*
 *	Program name and version:
 */
#define PROG_NAME	"XInterface Monitor"
#define PROG_VERSION	"1.8.4"

#define PROG_VERSION_MAJOR	1
#define PROG_VERSION_MINOR	8
#define PROG_VERSION_RELEASE	4


/*
 *	Usage message:
 */
#define PROG_USAGE_MESG "\
Usage: xifmon [interface] [options] [GUI_options]\n\
\n\
    [interface] is the name of the interface to monitor,\n\
    this value by default is ppp0.\n\
\n\
    [options] can be any of the following:\n\
\n\
        --bandwidth <bps>       Specifies the interface link bandwidth\n\
                                in bytes per second.\n\
        --bandwidth_bits <bps>  Specifies the interface link bandwidth\n\
                                in bits per second.\n\
	--initialize <path>     Specifies the script file to initialize\n\
                                (connect) the interface.\n\
        --shutdown <path>       Specifies the script file to shutdown\n\
                                (disconnect) the interface.\n\
        --help                  Prints (this) help screen and exits.\n\
        --version               Prints version information and exits.\n\
\n\
    If the --initialize or --shutdown arguments are not specified then\n\
    the connect and disconnect buttons (respectivly) will not function.\n\
\n\
    [GUI_options] can be any options standard to your GUI, consult\n\
    your GUI's manual for available options.\n\
\n"


/*
 *	Copyright message:
 */
#define PROG_COPYRIGHT	"\
Copyright (C) 1997-2001 WolfPack Entertainment.\n\
This program is protected by international copyright laws and treaties,\n\
distribution and/or modification of this software in violation of the\n\
GNU Public License is strictly prohibited. Violators will be prosicuted\n\
to the fullest extent of the law."


/*
 *	Action operation codes:
 */
#define IMW_ACTION_NONE			0
#define IMW_ACTION_CONNECT		10
#define IMW_ACTION_DISCONNECT		11
#define IMW_ACTION_EXIT			12

#define IMW_ACTION_MAP_STATS		20



/*
 *	Draw amount codes:
 */
#define IMW_DRAW_AMOUNT_COMPLETE	0
#define IMW_DRAW_AMOUNT_LEDS		1	/* Load stat leds. */
#define IMW_DRAW_AMOUNT_IFSTATE		2	/* IF state lights. */

#define SW_DRAW_AMOUNT_COMPLETE		0
#define SW_DRAW_AMOUNT_GRAPH		1


/*
 *	Interface state codes:
 */
#define IF_STATE_OFF		0	/* Off or non-existant. */
#define IF_STATE_STARTUP	1	/* Interface is starting up. */
#define IF_STATE_ON		2	/* Interface is on. */


/*
 *	Intervals (in milliseconds):
 */
#define INT_IF_STATS	200
#define INT_IF_STATUS	5000



/*
 *	Global program runlevel:
 */
extern int runlevel;

/*
 *	Current time in milliseconds:
 */
extern time_t cur_millitime;


/*
 *	Timmings (in milliseconds):
 */
typedef struct {

	time_t	if_stats,
		if_status;

} next_struct;
extern next_struct next;

/*
 *	Images:
 */
typedef struct {

	image_t	*icon,
		*bkg,
		*led_cursor,
		*if_light_cursor,
		*exit_label,
		*connect_label,
		*disconnect_label,
		*stats_label;

	pixmap_t icon_pm;

} xifmon_image_struct;
extern xifmon_image_struct xifmon_image;


/*
 *	Fonts:
 */
typedef struct {

	font_t	*graph_font;	/* Font displayed in graph. */

} xifmon_font_struct;
extern xifmon_font_struct xifmon_font;



/*
 *	Colors:
 */
typedef struct {

	pixel_t	graph_bkg,
		graph_grid,
		graph_plot,
		graph_text;

} xifmon_color_struct;
extern xifmon_color_struct xifmon_color;

/*
 *	Interface Stats Window:
 */
typedef struct {

	char map_state;  
        char is_in_focus;
        int x, y;
        unsigned int width, height;

        win_t toplevel;
        pixmap_t toplevel_buf;

	u_int8_t	*graph_pt_rx,
			*graph_pt_sx;

	int		total_graph_pts_rx,
			total_graph_pts_sx;

	time_t		last_update;

	win_t		graph_win;

	prompt_window_struct	ip_prompt,
				bandwidth_prompt,

				total_rx_prompt,
				total_sx_prompt,

				errors_rx_prompt,
				errors_sx_prompt,

				packets_rx_prompt,
				packets_sx_prompt,

				lrqs_rx_prompt,
				lrqs_sx_prompt;

	push_button_struct	refresh_btn,
				close_btn;

} stats_win_struct;


/*
 *	Interface Monitor Window:
 */
typedef struct {

	char map_state;
	char is_in_focus;
	int x, y;
	unsigned int width, height;

	bool_t	in_drag;

	win_t toplevel;
	pixmap_t toplevel_buf;
	image_t *toplevel_img;

	push_button_struct	connect_btn,
				disconnect_btn,
				exit_btn,
				stats_btn;

	menu_struct		menu;


	/* Script paths. */
	char *connect_script;		/* Initialize/connect script. */
	char *disconnect_script;	/* Shutdown/disconnect script. */

	/* Interface name. */
	char *interface;


	/* Interface statistics. */
	int if_state;			/* One of IF_STATE_*. */

	int bandwidth;			/* In bits per second. */
#define IF_BANDWIDTH_UNITS_BYTES_PER_SECOND	0
#define IF_BANDWIDTH_UNITS_BITS_PER_SECOND	1
	int bandwidth_units;		/* Converts bandwidth to alternate
					 * units when displayed, specified
					 * by one of IF_BANDWIDTH_UNITS_*.
					 */

	unsigned long	rx_delta,	/* In bytes. */
			sx_delta;

	unsigned long	rx_total,	/* In bytes. */
			sx_total;

	time_t		cur_time,	/* In milliseconds. */
			last_time;

	/* Anonymous pointer to interface structure. */
	void *if_data;

	/* Stats window. */
	stats_win_struct stats_win;

} ifmon_win_struct;

extern ifmon_win_struct ifmon_win;



/* In imwcb.c */
extern int IMWMenuCB(void *ptr, int op_code);

extern int IMWConnectPBCB(void *ptr);
extern int IMWDisconnectPBCB(void *ptr);
extern int IMWExitPBCB(void *ptr);
extern int IMWStatsPBCB(void *ptr);


/* In imwevent.c */
extern int IMWInit(ifmon_win_struct *imw, int argc, char *argv[]);
extern void IMWDraw(ifmon_win_struct *imw, int amount);
extern int IMWManage(ifmon_win_struct *imw, event_t *event);
extern void IMWMap(ifmon_win_struct *imw);
extern void IMWUnmap(ifmon_win_struct *imw);
extern void IMWDestroy(ifmon_win_struct *imw);


/* In imwstats.c */
extern int IMWStatsInit(ifmon_win_struct *imw, const char *interface);
extern int IMWStatsFetch(ifmon_win_struct *imw);
extern int IMWStatsIFStatus(ifmon_win_struct *imw);
extern void IMWStatsShutdown(ifmon_win_struct *imw);


/* In main.c */
extern void XIFMHandleSignal(int s);
extern void XIFMResetTimmers();
extern int XIFMInit(int argc, char *argv[]);
extern void XIFMManage();
extern void XIFMShutdown();


/* In swevent.c */
extern int SWClosePBCB(void *ptr);

extern int SWInit(stats_win_struct *sw, int argc, char *argv[]);
extern void SWDraw(stats_win_struct *sw, int amount);
extern int SWManage(stats_win_struct *sw, event_t *event);
extern void SWMap(stats_win_struct *sw);
extern void SWUnmap(stats_win_struct *sw);
extern void SWDestroy(stats_win_struct *sw);


/* In timming.c */
extern long MilliTime(void);


#endif	/* XIFMON_H */
